/*
 *   Copyright (C) 2006
 *   Siraj Razick <siraj@kdemail.net>
 *   PhobosK <phobosk@mail.kbfx.org>
 *   Lahiru Lakmal <llahiru@gmail.com>
 *   see Also AUTHORS
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License version 2 as
 *   published by the Free Software Foundation
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include <dcopref.h>
#include <kapplication.h>
#include <kdebug.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <krun.h>
#include <kservice.h>
#include <kservicegroup.h>
#include <kbfxconfig.h>
#include <kbfxplasmadataplugin-common.h>
#include <qdom.h>
#include <qfile.h>
#include <kdesktopfile.h>

#include <kstandarddirs.h>
#include <qdir.h>
#include <qfileinfo.h>

#include "tom.h"
#include "tomitem.h"
#include "tomtaskgroup.h"
#include "tomtaskaction.h"

Tom
parseXMLwithDOM( QDomDocument &d ){

    Tom m_tom;

    QDomElement tom = d.namedItem("tom").toElement();

    QDomNode tg = tom.firstChild();
    for ( ; !tg.isNull(); tg = tg.nextSibling() ) {
    	if ( tg.isElement() && tg.toElement().tagName() == "task_group" ) {

		TomTaskGroup* tomTaskGroup = new TomTaskGroup();
    		QDomElement tg_e = tg.toElement();
		QDomElement task_group_name = tg_e.namedItem("name").toElement();
    		//qDebug ( "Task_group name: " + task_group_name.text() + "\n");
		tomTaskGroup->setName(task_group_name.text());

		QDomNode task_ac = tg_e.firstChild();
    		for ( ; !task_ac.isNull(); task_ac = task_ac.nextSibling() ) {
    			if ( task_ac.isElement() && task_ac.toElement().tagName() == "task_action" ) {

				TomTaskAction* tomTaskAction = new TomTaskAction();
    				QDomElement task_ac_e = task_ac.toElement();
				QDomElement task_ac_name = task_ac_e.namedItem("name").toElement();
    				//qDebug ( "Task_action name: " + task_ac_name.text() + "\n");
				tomTaskAction->setName(task_ac_name.text());

				QDomNode item = task_ac_e.firstChild();
    				for ( ; !item.isNull(); item = item.nextSibling() ) {
    					if ( item.isElement() && item.toElement().tagName() == "item" ) {

						TomItem* tomItem = new TomItem();
    						QDomElement item_e = item.toElement();

						QDomElement desktop_file = item_e.namedItem("desktop_file").toElement();
    						//qDebug ( "Desktop file: " + desktop_file.text() + "\n");
						KDesktopFile *desktop = new KDesktopFile ( desktop_file.text() );

    						//qDebug ( "Item Program: " + desktop->readPath() + "\n");
						tomItem->setProgram(desktop->readPath());

    						//qDebug ( "Item Name: " + desktop->readName() + "\n");
						tomItem->setName(desktop->readName());

    						//qDebug ( "Item Desc: " + desktop->readDevice() + "\n");
						tomItem->setDesc(desktop->readDevice());

						//qDebug ( "Item Icon: " + desktop->readIcon() + "\n");
						tomItem->setIcon(desktop->readIcon());

						tomTaskAction->tomItems.append(tomItem);
					}
    				}

				tomTaskGroup->tomTaskActions.append(tomTaskAction);
			}
    		}
		m_tom.tomTaskGroups.append(tomTaskGroup);
	}
    }

    return m_tom;
}

KbfxDataStack *
view ()
{
	KbfxDataGroup *appGroup = new KbfxDataGroup ();
	KbfxDataGroupList *glist = new KbfxDataGroupList ();
	KbfxDataStack *gstack = new KbfxDataStack ();

	QFile f( "./tom.xml" );
	QDomDocument d;
	QString errorMsg;
	int errorLine, errorColumn;
	Tom tom;
	
	d.setContent( &f, &errorMsg, &errorLine, &errorColumn );
	
	tom = parseXMLwithDOM(d);
	
	QList<TomTaskGroup> tomTaskGroups_list = tom.tomTaskGroups;

	//list.setAutoDelete( TRUE ); // the list owns the objects

	if ( tomTaskGroups_list.count() <= 0)
	{
        	return new KbfxDataStack();
	}

	TomTaskGroup *tg;
	for ( tg = tomTaskGroups_list.first(); tg; tg = tomTaskGroups_list.next() ) {

		glist = new KbfxDataGroupList ();
		glist->setName ( tg->getName() );
		glist->setIcon ( "tom" ); //Icon Should Change

		QList<TomTaskAction> tomTaskActions_list = tg->tomTaskActions;

		TomTaskAction *ta;
		for ( ta = tomTaskActions_list.first(); ta; ta = tomTaskActions_list.next() ) {

			appGroup = new KbfxDataGroup ();
			appGroup->setName ( ta->getName() );

			QList<TomItem> tomItems_list = ta->tomItems;

			TomItem *t_item;
			for ( t_item = tomItems_list.first(); t_item; t_item = tomItems_list.next() ) {

				KbfxDataSource *src = new KbfxDataSource ();

				src->setName ( t_item->getName() );
				//src->setContentPath ( fi->filePath () );
				src->setIconPath ( t_item->getIcon() );
				//src->setCommand ( "dcop kicker Panel addApplet " + fi->filePath () );
				appGroup->addItem ( src );
			}

			glist->addGroup ( appGroup );
		}

		gstack->addGroupList ( glist );
	}

	gstack->setName ( "Tom" );


	return gstack;

}

QString
name ()
{
	return QString ( "TOM" );
}

QString
type ()
{
	return QString ( "Stub Type" );
}

uint
id ()
{
	return 1;			//TODO see what can be done... manybe rand()?
}

KbfxDataGroup *
search ( QString _keyword )
{
	_keyword = _keyword;
	return new KbfxDataGroup();

}
